
//not server only so the client also gets the game event setup stuff

#include "GameplayEvents.as"
#include "ScrollCommon.as";	
#include "TradingCommon.as";
#include "Descriptions.as";
#include "MigrantCommon.as";

const int coinsOnDamageAdd = 2;
const int coinsOnKillAdd = 20;

const int coinsOnDeathLosePercent = 20; //<100 coins = gaining coins if you keep a 1:1 k:d
const int coinsOnTKLose = 50;

const int coinsOnRestartAdd = 100;

string[] names;

void GiveRestartCoins(CPlayer@ p)
{
	p.server_setCoins( p.getCoins() + coinsOnRestartAdd);
}

void GiveRestartCoinsIfNeeded(CPlayer@ player)
{
	const string s = player.getUsername();
	for(uint i = 0; i < names.length; ++i)
	{
		if(names[i] == s)
		{
			return;
		}
	}
	
	names.push_back(s);
	GiveRestartCoins(player);
}

//extra coins on start to prevent stagnant round start
void onRestart(CRules@ this)
{
	if(!getNet().isServer())
		return;
	
	names.clear();
	
    uint count = getPlayerCount();
    for(uint p_step = 0; p_step < count; ++p_step)
    {
		CPlayer@ p = getPlayer(p_step);
		GiveRestartCoins( p );
		names.push_back(p.getUsername());
	}
}

//also given when plugging player -> on first spawn
void onSetPlayer( CRules@ this, CBlob@ blob, CPlayer@ player )
{
	if(!getNet().isServer())
		return;
		
	if(player !is null)
	{
		GiveRestartCoinsIfNeeded(player);
	}
}

//
// give coins for killing

void onPlayerDie( CRules@ this, CPlayer@ victim, CPlayer@ killer, u8 customData )
{
	if(!getNet().isServer())
		return;
	
	if (victim !is null )
	{
		if (killer !is null)
		{
			if (killer !is victim && killer.getTeamNum() != victim.getTeamNum())
			{
				killer.server_setCoins( killer.getCoins() + coinsOnKillAdd );
			}
			else if(killer.getTeamNum() == victim.getTeamNum())
			{
				killer.server_setCoins( killer.getCoins() - coinsOnTKLose );
			}
		}

		s32 lost = victim.getCoins() * (coinsOnDeathLosePercent*0.01f);

		victim.server_setCoins( victim.getCoins() - lost );
		
		//drop coins
		CBlob@ blob = victim.getBlob();
		if(blob !is null)
			server_DropCoins( blob.getPosition(), XORRandom(lost) );
	}
}

// give coins for damage

f32 onPlayerTakeDamage( CRules@ this, CPlayer@ victim, CPlayer@ attacker, f32 DamageScale )
{
	if(!getNet().isServer())
		return DamageScale;
	
	if (attacker !is null && attacker !is victim) {
		attacker.server_setCoins( attacker.getCoins() + DamageScale*coinsOnDamageAdd/this.attackdamage_modifier );
	}

	return DamageScale;
}

// coins for various game events
void onCommand( CRules@ this, u8 cmd, CBitStream @params )
{
	//only important on server
	if(!getNet().isServer())
		return;
	
	if (cmd == getGameplayEventID(this))
	{
		GameplayEvent g(params);
		
		u32 coins = 0;
		
		switch (g.getType())
		{
		case GE_built_block:
		
			{
				g.params.ResetBitIndex();
				u16 tile = g.params.read_u16();
				if(tile == CMap::tile_castle || tile == CMap::tile_wood)
				{
					if(this.isWarmup())
						coins = 1;
					else
						coins = 3;
				}
			}
			
			break;
			
		case GE_captured_flag:
			coins = 100;
			break;
		}
		
		if(coins > 0)
		{
			CPlayer@ p = g.getPlayer();
			if(p !is null)
			{
				p.server_setCoins( p.getCoins() + coins );
			}
		}
	}
}

// TRADING

void onBlobCreated( CRules@ this, CBlob@ blob )
{
	if (blob.getName() == "trader")
	{
		MakeWarTradeMenu( blob );
	}
}

TradeItem@ addGoldForItem( CBlob@ this, const string &in name,
							int cost, const string &in cost_mat, const string &in cost_mat_description, 
							const bool instantShipping,
							const string &in iconName,
							const string &in configFilename,
							const string &in description )
{
	TradeItem@ item = addTradeItem( this, name, 0, instantShipping, iconName, configFilename, description );
	if (item !is null && cost > 0) 
	{
		AddRequirement( item.reqs, "blob", cost_mat, cost_mat_description, cost );
		item.buyIntoInventory = true;
	}
	return item;
}

void MakeWarTradeMenu( CBlob@ trader )
{		
//	build menu
	 CreateTradeMenu( trader, Vec2f(2,1), "Trade" );

	
	f32 wood_sell_price = 5.0f;
	f32 wood_buy_price = 4.0f;
	
	f32 stone_sell_price = 2.0f;
	f32 stone_buy_price = 2.0f;
	
	s32 wood_stack_price = s32(500/wood_sell_price);
	s32 gold_wood_price = s32(500*wood_buy_price);
	
	s32 stone_stack_price = s32(500.0f/stone_sell_price);
	s32 gold_stone_price = s32(500.0f*stone_buy_price);
	
	/* 
	//addTradeEmptyItem( trader );
	addGoldForItem( trader, "Gold", gold_wood_price, "mat_wood", "Wood", true, 	"$mat_gold$", "mat_gold", "Exchange "+gold_wood_price+" Wood for 250 Gold" );
	//addGoldForItem( trader, "Gold", gold_stone_price, "mat_stone", "Stone", true, "$mat_gold$", "mat_gold", "Exchange "+gold_stone_price+" Stone for 250 Gold" );
    addTradeItem( trader, "Stone", stone_stack_price, true,						"$mat_stone$", "mat_stone", "Exchange "+stone_stack_price+" Gold for 250 Stone" );
	 */
}